#ifndef __LIBFWB_H
#define __LIBFWB_H

#if __GNUC__ > 5
#pragma GCC diagnostic ignored "-Wmisleading-indentation"
#endif

#ifndef _GNU_SOURCE
#define _GNU_SOURCE
#endif
#include <features.h>

#include <stdint.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/syscall.h>
#include <sys/mman.h>
#include <paths.h>
#include <errno.h>
#include "buffer.h"


/* gcc >= 3 is needed */
#ifndef __expect
#define __expect(foo,bar) __builtin_expect((long)(foo),bar)
#endif
#ifndef __likely
/* macros taken from Linux kernel */
#define __likely(foo) __expect((foo),1)
#define __unlikely(foo) __expect((foo),0)
#endif

/* DJB stuff */
void *alloc(size_t n);
void alloc_free(char *x);

long get_HZ();
const char *proc_stat_value(const char *s,int index);
#define PROC_PID_STAT_PID 0
#define PROC_PID_STAT_COMM 1
#define PROC_PID_STAT_STATE 2
#define PROC_PID_STAT_PPID 3
#define PROC_PID_STAT_PGRP 4
#define PROC_PID_STAT_SESSION 5
#define PROC_PID_STAT_TTY_NR 6
#define PROC_PID_STAT_TPGID 7
#define PROC_PID_STAT_FLAGS 8
#define PROC_PID_STAT_MINFLT 9
#define PROC_PID_STAT_CMINFLT 10
#define PROC_PID_STAT_MAJFLT 11
#define PROC_PID_STAT_CMAJFLT 12
#define PROC_PID_STAT_UTIME 13
#define PROC_PID_STAT_STIME 14
#define PROC_PID_STAT_CUTIME 15
#define PROC_PID_STAT_CSTIME 16
#define PROC_PID_STAT_PRIORITY 17
#define PROC_PID_STAT_NICE 18
#define PROC_PID_STAT_NUM_THREADS 19
#define PROC_PID_STAT_ITREALVALUE 20
#define PROC_PID_STAT_STARTTIME 21
#define PROC_PID_STAT_VSIZE 22
#define PROC_PID_STAT_RSS 23
#define PROC_PID_STAT_RSSLIM 24
#define PROC_PID_STAT_STARTCODE 25
#define PROC_PID_STAT_ENDCODE 26
#define PROC_PID_STAT_STARTSTACK 27
#define PROC_PID_STAT_KSTKESP 28
#define PROC_PID_STAT_KSTKEIP 29
#define PROC_PID_STAT_SIGNAL 30
#define PROC_PID_STAT_BLOCKED 31
#define PROC_PID_STAT_SIGIGNORE 32
#define PROC_PID_STAT_SIGCATCH 33
#define PROC_PID_STAT_WCHAN 34
#define PROC_PID_STAT_NSWAP 35
#define PROC_PID_STAT_CNSWAP 36
#define PROC_PID_STAT_EXIT_SIGNAL 37
#define PROC_PID_STAT_PROCESSOR 38
#define PROC_PID_STAT_RT_PRIORITY 39
#define PROC_PID_STAT_POLICY 40
#define PROC_PID_STAT_DELAYACCT_BLKIO_TICKS 41
#define PROC_PID_STAT_GUEST_TIME 42
#define PROC_PID_STAT_CGUEST_TIME 43
#define PROC_STAT_CPU_USER 1
#define PROC_STAT_CPU_NICE 2
#define PROC_STAT_CPU_SYSTEM 3
#define PROC_STAT_CPU_IDLE 4
#define PROC_STAT_CPU_IOWAIT 5
#define PROC_STAT_CPU_IRQ 6
#define PROC_STAT_CPU_SOFTIRQ 7
#define PROC_STAT_CPU_STEAL 8
#define PROC_STAT_CPU_GUEST 9

int32_t field_index(const char *s,int index) __attribute__((__deprecated__));
int32_t field_indexc(const char *s,int index, char sep, int multiple) __attribute__((__deprecated__));

void putlong(uint32_t num);
uint32_t sputlong(register char *buf, uint32_t num);
char *sputlongs(register char *buf, uint64_t num);
char *sputuint32(register char *buf, uint32_t num);
char *sputuint64(char *buf, uint64_t num);
#ifdef _USE_DEPRECATED_STR2UINT63
/*
#warning str2uint63() is deprecated, str2uint64() is recommended
*/
int64_t str2uint63(register const char *s) __attribute__((__deprecated__));
#endif
char *str2uint32(register char *s, uint32_t *val);
const char *str2uint64(register const char *s, uint64_t *val);
uint32_t str_cat(register char* out, register const char* in);
char *str_catc(register char* out, register const char c);
uint32_t str_catn(register char *out,register const char *in, uint32_t limit);
int32_t str_chr(register const char *in, register char needle);
int str_diff(register const char* s, register const char* t);
int str_diffn(register const char* s, register const char* t, uint32_t limit);
int str_diff_right(const char *u, const char *v);
char *str_field(const char *s,int index, char sep, int multiple);
int32_t str_index(const char* a, const char* b);
int32_t str_index2(const char* a, const char* b);
uint32_t str_len(register const char* in);

char *hexstr2uint32(register char *s, uint32_t *val);

const char *get_home_from_passwd(const char *pwdbuf, off_t pwdlen, uint32_t uid, uint32_t *userhome_len);
const char *get_name_from_passwd(const char *pwdbuf, off_t pwdlen, uint32_t uid, int *username_len);
const char *get_uid_from_passwd(register const char *pwdbuf, off_t pwdlen, const char *username, int username_len, int *uid_len);

const char *map_passwd_file(const char **pwdbuf, off_t *pwdlen);
int unmap_passwd_file(const char **pwdbuf, off_t *pwdlen);
const char *map_file(const char *fname, const char **fbuf, off_t *flen);

int write2file(const char *filename, register const char *str, register int len);
ssize_t file2buf(const char *filename, register char *buf, int size);

char *procnetipaddr_to_humanreadable(const char *procnets, char *ips);

/* also available as assembler source code: */
char *str_chrnul(register char* t, register char c);

/* 32-bit uid function mappings */
#ifndef __dietlibc
#ifndef uid32_t
#define uid32_t uint32_t
#endif
#ifndef gid32_t
#define gid32_t uint32_t
#endif
#endif
int chown32m(const char *path, uid32_t owner, gid32_t group);
int fchown32m(int fd, uid32_t owner, gid32_t group);
int lchown32m(const char *path, uid32_t owner, gid32_t group);
uid32_t getuid32m(void);
uid32_t geteuid32m(void);
gid32_t getgid32m(void);
gid32_t getegid32m(void);
int setuid32m(uid32_t uid);
int setgid32m(gid32_t gid);
int setreuid32m(uid32_t ruid, uid32_t euid);
int setregid32m(gid32_t rgid, gid32_t egid);
#define seteuid32m(euid) setreuid32m(-1,euid)
#define setegid32m(egid) setregid32m(-1,egid)
int getgroups32m(int size, gid32_t list[]);
int getresuid32m(uid32_t *ruid, uid32_t *euid, uid32_t *suid);
int getresgid32m(gid32_t *rgid, gid32_t *egid, gid32_t *sgid);
int setresuid32m(uid32_t ruid, uid32_t euid, uid32_t suid);
int setresgid32m(gid32_t rgid, gid32_t egid, gid32_t sgid);
/*extern inline void write1(const void *buf, size_t count);*/
#define write1(b,c) (syscall(__NR_write, 1, b, c))
#define write2(b,c) (syscall(__NR_write, 2, b, c))
/* It was Drepper again doing this crazy stuff with unused result warnings, wasn't he? */
#define chdirv(p) (syscall(__NR_chdir, p))

#endif
