#include "buffer.h"

#define BUFFER_CATFROMFILE_BUFSIZE 65536

/**
 * Copy the contents of a file to a write buffer.
 *
 * On issues with the arguments -1 is returned. On issues with the file or any other I/O error it will return -2.
 *
 * @param b Pointer to the write buffer.
 * @param filename The path/name of the file which contents should be pasted.
 * @return On success 0 is returned.
 */
int buffer_catfromfile(buffer *b, const char *filename) {
  register int c = -1;
  char buf[BUFFER_CATFROMFILE_BUFSIZE+1];
  register int bytes_read;
  int ret_value = -2;

  if ((b == 0) ||
      (b->s == 0) ||
      (b->l != 0) ||
      (b->fd == -1) ||
      (filename == 0) ||
      (*filename == 0)) return c;

  c = open(filename, O_RDONLY);
  if (c == -1) return ret_value;

  for (;;) {
    bytes_read = read(c, buf, BUFFER_CATFROMFILE_BUFSIZE);
    if (bytes_read == -1) goto out;
    if (bytes_read == 0) break;
    if (buffer_putmem(b, buf, bytes_read) == -1) goto out;
  }
  ret_value = 0;

  out:
  (void) close(c);
  return ret_value;
}

