#include "buffer.h"

#define BUF_SIZE 16384

/**
 * buffer_filecmp: compare two files like "cmp"
 *
 * @param filename1 The name of the first file.
 * @param filename2 The name of the second file.
 * @return -1 on error. 0 if both files have same content, 1 if files differ.
 */
int buffer_filecmp(const char *filename1, const char *filename2) {
  register int c1;
  register int c2;
  int ret_value = -1;
  char buf1[BUF_SIZE];
  char buf2[BUF_SIZE];
  buffer b1, b2;

  buffer_initread(&b1, buf1, BUF_SIZE, 0);
  buffer_initread(&b2, buf2, BUF_SIZE, 0);

  if (buffer_openread(&b1, filename1) == -1) return ret_value;
  if (buffer_openread(&b2, filename2) == -1) goto out2;

  for (;;) {
    c1 = buffer_getchar(&b1);
    if (c1 == -2) goto out1;
    c2 = buffer_getchar(&b2);
    if (c2 == -2) goto out1;
    if (c1 != c2) goto differ;
    if (c1 == EOF) goto equal;
  }

  equal:
  ret_value = 0;
  goto out1;

  differ:
  ret_value = 1;

  out1:
  buffer_close(&b2);
  out2:
  buffer_close(&b1);
  return ret_value;
}

