#include "buffer.h"

/**
 * Copy memory contents to a write buffer.
 *
 * On issues with the arguments -1 is returned. On issues with the file or any other I/O error it will return -2.
 *
 * @param b Pointer to the write buffer.
 * @param a Pointer to the memory to be pasted.
 * @param l Length of the memory content.
 * @return On success 0 is returned.
 */
int buffer_putmem(buffer *b, register const char *a, register uint32_t l) {
  register char *p;
  register char *s;
  register const char *t;

  if ((b == 0) ||
      (b->s == 0) ||
      (b->l != 0) ||
      (b->fd == -1) ||
      (a == 0)) return -1;

  if (l == 0) return 0;
  p = b->c;
  t = b->t;
  if ((p + l) <= t) goto put;

  s = b->s;
  if (write(b->fd, s, p-s) < 0) return -1;
  p = s;
  if ((p + l) >= t) {
    b->c = p;
    if (write(b->fd, a, l) < 0) return -1;
    return 0;
  }

  put:
  t = a + l;
  for(;;) {
    *p = *a; ++p; ++a; if (a >= t) break;
    *p = *a; ++p; ++a; if (a >= t) break;
    *p = *a; ++p; ++a; if (a >= t) break;
    *p = *a; ++p; ++a; if (a >= t) break;
  }
  b->c = p;
  return 0;
}
