#include "libfwb.h"

/**
 * $Id$
 */

/**
 * Open a file and read (one time only) data to specified buf.
 *
 * @param filename The name of the file the buf should be read from.
 * @param buf The buf to be filled from file.
 * @param size The size of the buf.
 * @return -1 on error. 0 if no bytes were read. On success return bytes read.
 */
ssize_t file2buf(const char *filename, register char *buf, int size) {
  register int c = -1;
  register ssize_t bytes_read;
  register ssize_t bytes_read_total;

  if (!buf) return c;
  *buf = 0;
  if (!filename) return c;
  if (!*filename) return c;
  if (size < 1) return c;
  
  c = open(filename, O_RDONLY);
  if (c == -1) return c;

  bytes_read_total = 0;
  for (;;) {
    bytes_read = read(c, buf, size);
    if (bytes_read <= 0) break;
    bytes_read_total += bytes_read;
    if (bytes_read == size) break;
    buf += bytes_read;
    size -= bytes_read;
  }
  (void) close(c);
  buf[bytes_read] = 0;
  return bytes_read_total;
}

