#include "libfwb.h"

#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

long get_HZ() {
  char buf[512];
  int fd;
  ssize_t c;
  long jiffies_per_second;
  uint64_t start_time;
  uint64_t val64;
  const char *p;

  /* try to calculate HZ value, don't trust some sysconf implementations */
  /* use /proc/self/stat, alternative method would be /proc/stat and uptime */
  fd = open("/proc/self/stat", O_RDONLY);
  if (fd == -1) goto sysconf_fallback;
  c = read(fd, buf, 512);
  close(fd);
  if (c == -1) goto sysconf_fallback;
  buf[c] = '\0';
  /* starttime */
  p = proc_stat_value(buf, PROC_PID_STAT_STARTTIME);
  if (p == 0) goto sysconf_fallback;
  str2uint64(p, &val64); /* no error checks */
  start_time = val64;
  fd = open("/proc/uptime", O_RDONLY);
  if (fd == -1) goto sysconf_fallback;
  c = read(fd, buf, 512);
  close(fd);
  if (c == -1) goto sysconf_fallback;
  buf[c] = '\0';
  /* uptime */
  str2uint64(buf, &val64); /* no error checks */
  jiffies_per_second = ((int)(start_time / val64)) + 1;
  /**
   * 10 S/390 (sometimes)
   * 20 User-mode Linux
   * 32 ia64 emulator
   * 64 StrongARM /Shark
   * 100 normal Linux
   * 128 MIPS, ARM
   * 1000 ARM
   * 1024 Alpha, ia64
   * 1200 Alpha
   */
  jiffies_per_second = ((int)(jiffies_per_second / 2)) * 2; /* HZ */
  return jiffies_per_second;

  sysconf_fallback:
  return sysconf(_SC_CLK_TCK);
}

