#include "libfwb.h"

/**
 * Return index to a field in a string, most probably from a proc stat file.
 *
 * It searches a string for a field specified by number
 * 'index' and returns a pointer to this field.
 * One space (" ") is taken as field separator.
 * If the end of the string is reached without finding
 * field number 'index' then 0 is returned.
 * For /proc/[pid]/stat searching the constants starting
 * with PROC_STAT_ can be used.
 *
 * @param s The string to be searched through.
 * @param index The number of the field index to search for.
 * @return A pointer to the field in the string or 0 on error.
 */
const char *proc_stat_value(const char *s,int index) {
  register const char* t = s;

  if (!t) return 0;
  if (index < 0) return 0;
  if (index == 0) return s;
  for (;;) {
    if (!*t) return 0; if (*t == ' ') { --index; if (!index) { ++t; return t; } } ++t;
    if (!*t) return 0; if (*t == ' ') { --index; if (!index) { ++t; return t; } } ++t;
    if (!*t) return 0; if (*t == ' ') { --index; if (!index) { ++t; return t; } } ++t;
    if (!*t) return 0; if (*t == ' ') { --index; if (!index) { ++t; return t; } } ++t;
  }
}
