#include "libfwb.h"

/**
 * Convert number from string to (unsigned) integer.
 *
 * The given string is parsed for ciphers '0' - '9'.
 * If the first char is not a cipher then 0 will be returned.
 * Parsing will stop at the first non-cipher char.
 * No boundary and carry checks are made.
 * The second arg is a pointer to the var to save the result in.
 * This pointer may be 0. This can be used to just return a
 * pointer to the char after the number.
 *
 * $Id$
 *
 * @param s The string to parse for a number.
 * @param val a pointer to the variable to store the value or NULL
 * @return a pointer to the first char after number or 0 on error
 */
const char *str2uint64(register const char *s, uint64_t *val) {
  register char c;
  uint64_t i;

  if (val == 0) return 0;
  *val = 0;
  c = *s;
  if ((c < '0') || (c > '9')) return 0;
  ++s;
  i = c - '0';
  for(;;) {
    c = *s; if ((c < '0') || (c > '9')) break; i = (i * 10) + c - '0'; ++s;
    c = *s; if ((c < '0') || (c > '9')) break; i = (i * 10) + c - '0'; ++s;
    c = *s; if ((c < '0') || (c > '9')) break; i = (i * 10) + c - '0'; ++s;
    c = *s; if ((c < '0') || (c > '9')) break; i = (i * 10) + c - '0'; ++s;
  }
  *val = i;
  return s;
}
