#include <stdio.h>
#include <stdlib.h>
#include <sput.h>
#include <buffer.h>

/** @file
 * \brief Unit tests of buffer library
 */

#define TEST_BUFFER_CREATE_MODE O_RDWR|O_CREAT|O_TRUNC
#define TEST_BUFFER_FILE_PERMISSIONS S_IRUSR|S_IWUSR|S_IRGRP|S_IROTH

static int compare_string_to_file_content(const char *string, const char *filename);

/*
 * EVIL! This is just a workaround for SPUT macros.
 * No single object file built by Makefile.
 */
#include "buffer_close.c"
#include "buffer_flush.c"
#include "buffer_flushlines.c"
#include "buffer_openread.c"
#include "buffer_openwrite.c"
#include "buffer_putc.c"
#include "buffer_putmem.c"
#include "buffer_putstr.c"
#include "buffer_catfromfile.c"
#include "buffer_putuint32.c"
#include "buffer_putuint64.c"
#include "buffer_putuint8.c"
#include "buffer_getchar.c"
#include "buffer_getline.c"

int main(int argc, char *argv[])
{
    sput_start_testing();

    sput_enter_suite("buffer, buffer_close()");
    sput_run_test(test_buffer_close);

    sput_enter_suite("buffer, buffer_flush()");
    sput_run_test(test_buffer_flush);

    sput_enter_suite("buffer, buffer_flushlines()");
    sput_run_test(test_buffer_flushlines);

    sput_enter_suite("buffer, buffer_openread()");
    sput_run_test(test_buffer_openread);

    sput_enter_suite("buffer, buffer_openwrite()");
    sput_run_test(test_buffer_openwrite);

    sput_enter_suite("buffer, buffer_putc()");
    sput_run_test(test_buffer_putc);

    sput_enter_suite("buffer, buffer_putmem()");
    sput_run_test(test_buffer_putmem);

    sput_enter_suite("buffer, buffer_putstr()");
    sput_run_test(test_buffer_putstr);

    sput_enter_suite("buffer, buffer_catfromfile()");
    sput_run_test(test_buffer_catfromfile);

    sput_enter_suite("buffer, buffer_putuint32()");
    sput_run_test(test_buffer_putuint32);

    sput_enter_suite("buffer, buffer_putuint64()");
    sput_run_test(test_buffer_putuint64);

    sput_enter_suite("buffer, buffer_putuint8()");
    sput_run_test(test_buffer_putuint8);

    sput_enter_suite("buffer, buffer_getchar()");
    sput_run_test(test_buffer_getchar);

    sput_enter_suite("buffer, buffer_getline()");
    sput_run_test(test_buffer_getline);

    sput_finish_testing();

    return sput_get_return_value();
}

static int compare_string_to_file_content(const char *string, const char *filename) {
  char buf[256];
  int fd;
  size_t read_bytes;

  fd = open(filename, O_RDONLY);
  if (fd < 0) return -1;
  read_bytes = read(fd, buf, 255);
  if (read_bytes < 0) return -1;
  buf[read_bytes] = 0;
  return strcmp(string, buf);
}
