#define TEST_BUFFER_CATFROMFILE_BUFSIZE 2

static void test_buffer_catfromfile() {
  int fd;
  const char tempfilename[]="tests/test_tempfile_buffer_catfromfile";
  const char filename[]="tests/testfile_buffer_catfromfile";
  const char not_existing_filename[]="9e874a86677e6e8b012c82acc4c166cf";
  char buf1space[TEST_BUFFER_CATFROMFILE_BUFSIZE];
  char buf1xspace[255];
  buffer buf1;

  /* initializing */
  buffer_initwrite(&buf1, buf1space, TEST_BUFFER_CATFROMFILE_BUFSIZE, 1);

  fd = open(filename, O_RDONLY);
  sput_fail_unless(fd >= 0, "open testfile with data");
  close(fd);

  fd = open(tempfilename, TEST_BUFFER_CREATE_MODE, TEST_BUFFER_FILE_PERMISSIONS);
  sput_fail_unless(fd >= 0, "open test tempfile");

  /* tests */

  /* simple tests of arguments */
  sput_fail_unless(buffer_catfromfile(&buf1, 0) == -1, "filename is 0");
  *buf1space = 0;
  sput_fail_unless(buffer_catfromfile(&buf1, buf1space) == -1, "*filename is 0");
  sput_fail_unless(buffer_catfromfile(0, filename) == -1, "buffer is 0");
  buffer_initread(&buf1, buf1space, TEST_BUFFER_CATFROMFILE_BUFSIZE, fd);
  sput_fail_unless(buffer_catfromfile(&buf1, filename) == -1, "buffer is read buffer");
  buffer_initwrite(&buf1, buf1space, TEST_BUFFER_CATFROMFILE_BUFSIZE, -1);
  sput_fail_unless(buffer_catfromfile(&buf1, filename) == -1, "buffer has fd -1");
  buffer_initwrite(&buf1, buf1space, TEST_BUFFER_CATFROMFILE_BUFSIZE, fd);
  sput_fail_unless(buffer_catfromfile(&buf1, not_existing_filename) == -2, "cat file does not exist");

  /* functional tests */
  sput_fail_unless(buffer_catfromfile(&buf1, filename) == 0, "buffer_catfromfile() with empty buffer");
  buffer_flush(&buf1);
  close(fd);
  sput_fail_unless(compare_string_to_file_content("12345", tempfilename) == 0, "check output written");

  fd = open(tempfilename, TEST_BUFFER_CREATE_MODE, TEST_BUFFER_FILE_PERMISSIONS);
  buffer_initwrite(&buf1, buf1space, TEST_BUFFER_CATFROMFILE_BUFSIZE, fd);
  buffer_putc(&buf1,'0');
  sput_fail_unless(buffer_catfromfile(&buf1, filename) == 0, "buffer_catfromfile() with non-empty buffer");
  buffer_flush(&buf1);
  close(fd);
  sput_fail_unless(compare_string_to_file_content("012345", tempfilename) == 0, "check output written");

  fd = open(tempfilename, TEST_BUFFER_CREATE_MODE, TEST_BUFFER_FILE_PERMISSIONS);
  buffer_initwrite(&buf1, buf1space, TEST_BUFFER_CATFROMFILE_BUFSIZE, fd);
  buffer_catfromfile(&buf1, filename);
  sput_fail_unless(buffer_putc(&buf1,'6') == 0, "buffer_putc() after buffer_catfromfile()");
  buffer_flush(&buf1);
  close(fd);
  sput_fail_unless(compare_string_to_file_content("123456", tempfilename) == 0, "check output written");

  fd = open(tempfilename, TEST_BUFFER_CREATE_MODE, TEST_BUFFER_FILE_PERMISSIONS);
  buffer_initwrite(&buf1, buf1space, 1, fd);
  sput_fail_unless(buffer_catfromfile(&buf1, filename) == 0, "buffer_catfromfile() with empty buffer - 1 byte buffer");
  buffer_flush(&buf1);
  close(fd);
  sput_fail_unless(compare_string_to_file_content("12345", tempfilename) == 0, "check output written");

  fd = open(tempfilename, TEST_BUFFER_CREATE_MODE, TEST_BUFFER_FILE_PERMISSIONS);
  buffer_initwrite(&buf1, buf1space, TEST_BUFFER_CATFROMFILE_BUFSIZE, fd);
  buffer_putc(&buf1,'0');
  sput_fail_unless(buffer_catfromfile(&buf1, filename) == 0, "buffer_catfromfile() with non-empty buffer - 1 byte buffer");
  buffer_flush(&buf1);
  close(fd);
  sput_fail_unless(compare_string_to_file_content("012345", tempfilename) == 0, "check output written");

  fd = open(tempfilename, TEST_BUFFER_CREATE_MODE, TEST_BUFFER_FILE_PERMISSIONS);
  buffer_initwrite(&buf1, buf1space, TEST_BUFFER_CATFROMFILE_BUFSIZE, fd);
  buffer_catfromfile(&buf1, filename);
  sput_fail_unless(buffer_putc(&buf1,'6') == 0, "buffer_putc() after buffer_catfromfile() - 1 byte buffer");
  buffer_flush(&buf1);
  close(fd);
  sput_fail_unless(compare_string_to_file_content("123456", tempfilename) == 0, "check output written");

  fd = open(tempfilename, TEST_BUFFER_CREATE_MODE, TEST_BUFFER_FILE_PERMISSIONS);
  buffer_initwrite(&buf1, buf1xspace, 255, fd);
  sput_fail_unless(buffer_catfromfile(&buf1, filename) == 0, "buffer_catfromfile() with big empty buffer");
  buffer_flush(&buf1);
  close(fd);
  sput_fail_unless(compare_string_to_file_content("12345", tempfilename) == 0, "check output written");

  fd = open(tempfilename, TEST_BUFFER_CREATE_MODE, TEST_BUFFER_FILE_PERMISSIONS);
  buffer_initwrite(&buf1, buf1xspace, 255, fd);
  buffer_putc(&buf1,'0');
  sput_fail_unless(buffer_catfromfile(&buf1, filename) == 0, "buffer_catfromfile() with big non-empty buffer");
  buffer_flush(&buf1);
  close(fd);
  sput_fail_unless(compare_string_to_file_content("012345", tempfilename) == 0, "check output written");

}

